<?php

namespace FiloBlu\Refilo\Model\Adapter;

use FiloBlu\Refilo\Helper\Data as HelperData;
use MongoDB\Driver\BulkWrite;
use MongoDB\Driver\Command;
use MongoDB\Driver\Exception\Exception;
use MongoDB\Driver\Manager;

class Mongo extends AbstractAdapter
{
    /**
     * @var HelperData
     */
    private $helperData;

    /**
     * Mongo constructor.
     * @param HelperData $helperData
     */
    public function __construct(
        HelperData $helperData
    ) {
        $this->helperData = $helperData;
        $this->connect();
    }

    public function connect()
    {
//        $this->_connection = new Manager($this->helperData->getConfigValue("mongo/general/mongo_host"));
        $this->_connection = new Manager("mongodb://35.205.137.140:27018");
    }

    /**
     * @param $collection
     * @param $mongoDatabase
     * @throws Exception
     */
    public function dropCollection($collection, $mongoDatabase)
    {
        $collectionExist = !empty($this->_connection->executeCommand($mongoDatabase, new Command(['listCollections' => 1, "filter" => ["name" => $collection]]))->toArray());

        if ($collectionExist) {
            try {
                $this->_connection->executeCommand($mongoDatabase, new Command(["drop" => $collection]));
                if (php_sapi_name() === 'cli') {
                    printf("Dropped %s collection\n", "{$collection}");
                }
            } catch (\MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $key
     * @param $objects
     * @param $collection
     * @param bool $insertMode
     * @param $mongoDatabase
     * @return mixed|void
     */
    public function update($key, $objects, $collection, $insertMode, $mongoDatabase)
    {
        $bulk = new BulkWrite();

        foreach ($objects as $object) {
            $objToUpdate = $object;
            if ($insertMode === false) {
                $objToUpdate = ['$set' => $object];
            }
            try {
                $bulk->update(['_id' => $object->{$key}], $objToUpdate, ['multi' => false, 'upsert' => $insertMode]);
            } catch (MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        }

        try {
            $result = $this->_connection->executeBulkWrite(sprintf('%s.%s', $mongoDatabase, $collection), $bulk);
            if (php_sapi_name() === 'cli') {
                printf("Inserted %d document(s)\n", $result->getInsertedCount());
                printf("Matched  %d document(s)\n", $result->getMatchedCount());
                printf("Updated  %d document(s)\n", $result->getModifiedCount());
                printf("Upserted %d document(s)\n", $result->getUpsertedCount());
                printf("Deleted  %d document(s)\n", $result->getDeletedCount());
            }
        } catch (MongoDB\Driver\Exception\Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * @return mixed
     * @throws Exception
     */
    public function ensureIndex($fields,$collection)
    {
        try {

            $database = $this->helperData->getConfigValue('mongo/general/mongo_general_database');
            $indexes = [];

            foreach ($fields as $field) {

                $indexes[] = [
                    'name' => $field,
                    'key' => [$field => 1],
                    'ns' => "{$database}.{$collection}",
                ];

            }

            $command = new Command([
                'createIndexes' => $collection,
                'indexes' => $indexes,
            ]);

            $result = $this->_connection->executeCommand($database, $command);
            printf("added Indexes for %s \n", "{$database}.{$collection}");
        } catch (MongoDB\Driver\Exception\Exception $e) {
            echo $e->getMessage();
        }
    }
}
